using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Threading;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Delegate

        /// <summary>
        ///  ϱ 븮
        /// </summary>
        /// <param name="value"></param>
        private delegate void ChangeOpacityDelegate(double value);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 콺 Ż 
        /// </summary>
        private double mouseLeaveOpacity = 0.7;

        /// <summary>
        /// FADE 
        /// </summary>
        private double fadeRate = 0.001;

        /// <summary>
        /// FADE IN 
        /// </summary>
        private Thread fadeInThread;

        /// <summary>
        /// FADE OUT 
        /// </summary>
        private Thread fadeOutThread;

        /// <summary>
        ///  콺 ġ
        /// </summary>
        private Point lastPoint;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainForm()

        /// <summary>
        /// 
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Load                  += Form_Load;
            Paint                 += Form_Paint;
            MouseEnter            += Form_MouseEnter;
            MouseDown             += Form_MouseDown;
            MouseMove             += Form_MouseMove;
            MouseLeave            += Form_MouseLeave;
            this.exitButton.Click += exitButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region  ε óϱ - Form_Load(sender, e)

        /// <summary>
        ///  ε óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_Load(object sender, EventArgs e)
        {
            DoubleBuffered = true;

            TransparencyKey = Color.Lime;

            Opacity = mouseLeaveOpacity;

            this.fadeInThread  = new Thread(new ThreadStart(ProcessFadeIn ));
            this.fadeOutThread = new Thread(new ThreadStart(ProcessFadeOut));
        }

        #endregion
        #region  콺 Խ óϱ - Form_MouseEnter(sender, e)

        /// <summary>
        ///  콺 Խ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_MouseEnter(object sender, EventArgs e)
        {
            if(this.fadeInThread.ThreadState == ThreadState.Running)
            {
                this.fadeInThread.Abort();
            }

            if(this.fadeOutThread.ThreadState == ThreadState.Running)
            {
                fadeOutThread.Abort();
            }

            this.fadeInThread = new Thread(new ThreadStart(ProcessFadeIn));

            this.fadeInThread.Start();
        }

        #endregion
        #region  콺 DOWN óϱ - Form_MouseDown(sender, e)

        /// <summary>
        ///  콺 DOWN óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_MouseDown(object sender, MouseEventArgs e)
        {
            this.lastPoint = new Point(e.X, e.Y);
        }

        #endregion
        #region  콺 ̵ óϱ - Form_MouseMove(sender, e)

        /// <summary>
        ///  콺 ̵ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_MouseMove(object sender, MouseEventArgs e)
        {
            if(e.Button == MouseButtons.Left || e.Button == MouseButtons.Right)
            {
                Left += e.X - this.lastPoint.X;
                Top  += e.Y - this.lastPoint.Y;
            }
        }

        #endregion
        #region  콺 Ż óϱ - Form_MouseLeave(sender, e)

        /// <summary>
        ///  콺 Ż óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_MouseLeave(object sender, EventArgs e)
        {
            if(Bounds.Contains(Cursor.Position))
            {
                return;
            }

            if(this.fadeInThread.ThreadState == ThreadState.Running)
            {
                this.fadeInThread.Abort();
            }

            if(this.fadeOutThread.ThreadState == ThreadState.Running)
            {
                this.fadeOutThread.Abort();
            }

            this.fadeOutThread = new Thread(new ThreadStart(ProcessFadeOut));

            this.fadeOutThread.Start();
        }

        #endregion
        #region  Ʈ óϱ - Form_Paint(sender, e)

        /// <summary>
        ///  Ʈ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_Paint(object sender, PaintEventArgs e)
        {
            Graphics graphics = e.Graphics;

            graphics.SmoothingMode = SmoothingMode.AntiAlias;

            Rectangle areaRectangle = this.ClientRectangle;

            areaRectangle.Width  -= 1;
            areaRectangle.Height -= 1;

            using(Pen outlinePen = new Pen(Color.Gold, 1))
            {
                graphics.FillRectangle(Brushes.LightYellow, ClientRectangle);

                graphics.DrawRectangle(outlinePen, areaRectangle);
            }
        }

        #endregion
        #region  ư Ŭ óϱ - exitButton_Click(sender, e)

        /// <summary>
        ///  ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void exitButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region  ϱ - ChangeOpacity(value)

        /// <summary>
        ///  ϱ
        /// </summary>
        /// <param name="value"></param>
        private void ChangeOpacity(double value)
        {
            if(InvokeRequired)
            {
                ChangeOpacityDelegate changeOpacityDelegate = new ChangeOpacityDelegate(ChangeOpacity);

                this.Invoke(changeOpacityDelegate, value);
            }
            else
            {
                Opacity = value;
            }
        }

        #endregion

        #region FADE IN óϱ - ProcessFadeIn()

        /// <summary>
        /// FADE IN óϱ
        /// </summary>
        private void ProcessFadeIn()
        {
            for(double i = Opacity; i <= 1; i += this.fadeRate)
            {
                ChangeOpacity(i);
            }

            if(Opacity != 1.0)
            {
                ChangeOpacity(1.0);
            }
        }

        #endregion
        #region FADE OUT óϱ - ProcessFadeOut()

        /// <summary>
        /// FADE OUT óϱ
        /// </summary>
        private void ProcessFadeOut()
        {
            for(double i = Opacity; i >= this.mouseLeaveOpacity; i -= this.fadeRate)
            {
                ChangeOpacity(i);
            }

            if(Opacity != this.mouseLeaveOpacity)
            {
                ChangeOpacity(this.mouseLeaveOpacity);
            }
        }

        #endregion
    }
}